@@ -19,6 +19,7 @@
  */
 
 #include <config.h>
+#include <missing.h>
 
 #include <stdio.h>
 #include <ctype.h>
@@ -148,8 +149,6 @@ char *s_net_return_connected_string(TOPLEVEL * pr_current, OBJECT * object,
     char *temp_uref = NULL;
     char *string;
     char *misc;
-    int hierarchy_tag_len = 0;
-
 
     o_current = object;
 
@@ -162,13 +161,7 @@ char *s_net_return_connected_string(TOPLEVEL * pr_current, OBJECT * object,
     scm_uref = g_scm_c_get_uref(pr_current, o_current->parent);
 
     if (scm_is_string( scm_uref )) {
-      temp_uref = scm_to_locale_string( scm_uref );
-    }
-
-    if (hierarchy_tag) {
-	hierarchy_tag_len = strlen(hierarchy_tag)+1;
-    } else {
-	hierarchy_tag_len = 0;
+      temp_uref = scm_to_utf8_string (scm_uref);
     }
 
     /* apply the hierarchy name to the uref */
@@ -351,7 +344,6 @@ char *s_net_name (TOPLEVEL * pr_current, NETLIST * netlist_head,
     CPINLIST *pl_current;
     char *net_name = NULL;
     int found = 0;
-    int hierarchy_tag_len;
     char *temp;
     int *unnamed_counter;
     char *unnamed_string;
@@ -416,13 +408,6 @@ char *s_net_name (TOPLEVEL * pr_current, NETLIST * netlist_head,
 
     }
 
-    if (hierarchy_tag) {
-	/* + 1 is for the '/' character */
-	hierarchy_tag_len = strlen(hierarchy_tag) + 1;
-    } else {
-	hierarchy_tag_len = 0;
-    }
-
     switch (type) {
       case PIN_TYPE_NET:
         unnamed_counter = &unnamed_net_counter;
